'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

exports.default = loadAndProcess;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _each = require('lodash/each');

var _each2 = _interopRequireDefault(_each);

var _map = require('lodash/map');

var _map2 = _interopRequireDefault(_map);

var _omit = require('lodash/omit');

var _omit2 = _interopRequireDefault(_omit);

var _cloneDeep = require('lodash/cloneDeep');

var _cloneDeep2 = _interopRequireDefault(_cloneDeep);

var _isEmpty = require('lodash/isEmpty');

var _isEmpty2 = _interopRequireDefault(_isEmpty);

var _assignIn = require('lodash/assignIn');

var _assignIn2 = _interopRequireDefault(_assignIn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var debug = require('debug')('json-di');
var keywords = ['options', 'module', 'require'];
var extendProcessed = function extendProcessed(obj, data) {
  var toAssign = (0, _omit2.default)(data, keywords);

  if (!(0, _isEmpty2.default)(toAssign)) {
    debug('Extending result with additional data', obj, toAssign);
    return (0, _assignIn2.default)(obj, toAssign);
  }

  return obj;
};

function load(mod, parent) {
  var data = (0, _cloneDeep2.default)(mod || {});
  var name = data.require;

  var location = '';
  var loadedModule = void 0;

  // If the object has a `require` statement
  if (name) {
    var root = _path2.default.dirname(parent);
    debug('Loading require ' + name);

    // The location of the file to require
    try {
      location = require.resolve(name);
      debug('require.resolve for ' + name + ' returned ' + location);
    } catch (error) {
      location = _path2.default.isAbsolute(name) ? name : _path2.default.join(root, name);
      debug('resolved location for ' + name + ' to ' + location);
    }

    try {
      loadedModule = require(location);
    } catch (error) {
      throw new Error('Can not load module ' + name + ' defined in ' + parent + ' (`' + JSON.stringify(data) + '`)');
    }

    if (_path2.default.extname(name) === '.json') {
      // if it s a JSON configuration file, clone its data
      // (since we are modifying it and required JSON files are singletons)
      // and process it recursively
      debug('Recursively loading JSON configuration ' + name);
      data.module = load(loadedModule, location);
    } else {
      data.module = loadedModule;
    }
  }

  // Except for `require` keys, process all other keys
  (0, _each2.default)(data, function (value, key) {
    if ((typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' && key !== 'module') {
      data[key] = load(value, parent);
    }
  });

  return data;
}

function process(data) {
  var _this = this;

  var convert = arguments.length <= 1 || arguments[1] === undefined ? function (value) {
    return value;
  } : arguments[1];

  if ((typeof data === 'undefined' ? 'undefined' : _typeof(data)) !== 'object') {
    return Promise.resolve(data);
  }

  if (Array.isArray(data)) {
    // Map data arrays
    return Promise.all(data.map(function (current) {
      return process(current, convert);
    }));
  }

  var result = {};

  // Process all options, run the converter or process recursively
  return Promise.all((0, _map2.default)(data, function (value, key) {
    if (key !== 'module') {
      var processed = (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' ? process(value, convert) : Promise.resolve(convert(value, key, data));

      return processed.then(function (data) {
        return result[key] = data;
      });
    }

    return Promise.resolve();
  })).then(function () {
    var mod = data.module;

    // If this is a loaded module
    if (typeof mod !== 'undefined') {
      // If the module is a function and `options` or `arguments`
      // is specified in the configuration, run that function with options or arguments
      if (result.options && typeof mod === 'function') {
        var args = Array.isArray(result.options) ? result.options : [result.options];
        debug('Calling function returned from ' + data.require + ' with', args);

        // Call the module with the given arguments since it can return a
        // promise we'll always wrap it before extending the result with
        // any additional data
        return Promise.resolve(mod.call.apply(mod, [_this].concat(_toConsumableArray(args)))).then(function (modResult) {
          return extendProcessed(modResult, result);
        });
      }

      // If we are loading a JSON configuration file, process it recursively
      if (data.require && _path2.default.extname(data.require) === '.json') {
        debug('Recursively processing JSON configuration file ' + data.require);
        return process(mod, convert);
      }

      // Otherwise just return the module
      return mod;
    }

    return result;
  });
}

function loadAndProcess(data, parent, convert) {
  var loaded = load(data, parent);

  return process(loaded, convert);
}

(0, _assignIn2.default)(loadAndProcess, { load: load, process: process });
module.exports = exports['default'];